from pydantic import BaseModel, Field
from .snippet import Snippet
from .status import Status
from .statistics import Statistics
from .paid_product_placement_details import PaidProductPlacementDetails
from .player import Player
from .topic_details import TopicDetails
from .recording_details import RecordingDetails
from .file_details import FileDetails
from .processing_details import ProcessingDetails
from .suggestions import Suggestions
from .livestreaming_details import LiveStreamingDetails
from .localizations import Localizations
from typing import Optional


class Video(BaseModel):
    kind: Optional[str] = Field(default=None)
    etag: Optional[str] = Field(default=None)
    id: Optional[str] = Field(default=None)
    snippet: Optional[Snippet] = Field(default=None)
    status: Optional[Status] = Field(default=None)
    statistics: Optional[Statistics] = Field(default=None)
    paidProductPlacementDetails: Optional[PaidProductPlacementDetails] = Field(
        default=None
    )
    player: Optional[Player] = Field(default=None)
    topicDetails: Optional[TopicDetails] = Field(default=None)
    recordingDetails: Optional[RecordingDetails] = Field(default=None)
    fileDetails: Optional[FileDetails] = Field(default=None)
    processingDetails: Optional[ProcessingDetails] = Field(default=None)
    suggestions: Optional[Suggestions] = Field(default=None)
    liveStreamingDetails: Optional[LiveStreamingDetails] = Field(default=None)
    localizations: Optional[Localizations] = Field(default=None)



from pydantic import BaseModel, Field
from typing import Literal


class Status(BaseModel):
    uploadStatus: Literal["deleted", "failed", "processed", "rejected", "uploaded"]
    failureReason: Literal[
        "codec", "conversion", "invalidFile", "tooSmall", "uploadAborted"
    ]
    rejectionReason: Literal[
        "claim",
        "copyright",
        "duplicate",
        "inappropriate",
        "length",
        "termsOfUse",
        "trademark",
        "uploaderAccountClosed",
        "uploaderAccountSuspended",
    ]
    privacyStatus: Literal["private", "unlisted", "public"]
    publishAt: str
    failureReason: Literal["creativeCommon", "youtube"]
    embeddable: bool
    publicStatsViewable: bool
    madeForKids: bool
    selfDeclaredMadeForKids: bool
    containsSyntheticMedia: bool



from .authentication import MissingClientSecretsFile, InvalidSecretsFileError
from .video import VideoNotFoundException
from .authorization import ForbiddenError

__all__ = [
    "MissingClientSecretsFile",
    "InvalidSecretsFileError",
    "VideoNotFoundException",
    "ForbiddenError",
]



from typing import Optional

from pydantic import BaseModel


class Thumbnail(BaseModel):
    resolution: Optional[str] = None
    url: Optional[str] = None
    width: Optional[int] = None
    height: Optional[int] = None



from . import YouTube, MissingClientSecretsFile, InvalidSecretsFileError
import pytest
from pathlib import Path
from unittest.mock import patch


def test_secrets_file_not_provided():
    youtube = YouTube()
    with pytest.raises(MissingClientSecretsFile):
        youtube.authenticate()


def test_secrets_file_not_found():
    youtube = YouTube()
    with pytest.raises(FileNotFoundError):
        youtube.authenticate('client_secret.json')


def test_empty_secrets_file_provided():
    youtube = YouTube()
    with pytest.raises(InvalidSecretsFileError):
        youtube.authenticate('secret.json')


def test_youtube_authenticated():
    with patch.object(YouTube, 'authenticate', return_value=None) as authenticate:
        youtube = YouTube()
        assert youtube.authenticate('secret.json') is None
    authenticate.assert_called_once_with('secret.json')
    


from datetime import datetime
from typing import Literal

from pydantic import BaseModel, Field
from typing import Optional

from .localized import Localized
from .thumbnail import Thumbnail


class Snippet(BaseModel):
    title: Optional[str] = Field(default=None)
    description: Optional[str] = Field(default=None)
    thumbnails: list[Thumbnail] = []
    publishedAt: Optional[str] = Field(default=None)
    channelId: Optional[str] = Field(default=None)
    channelTitle: Optional[str] = Field(default=None)
    categoryId: Optional[str] = Field(default=None)
    categoryId: Optional[str] = Field(default=None)
    localized: Optional[Localized] = Field(default=None)
    defaultAudioLanguage: Optional[str] = Field(default=None)
    defaultLanguage: Optional[str] = Field(default=None)
    tags: list[str] = []



from gverify import GoogleOAuth, YouTubeScopes
from pydantic import BaseModel
from typing import Any, Optional
from .exceptions import MissingClientSecretsFile, InvalidSecretsFileError
from pathlib import Path
from .models import Video
from .resources import YouTubeVideoResource
from gverify.exceptions import InvalidSecretsFileException


class YouTube(BaseModel):
    """Provides methods for interacting with the YouTube API.

    This class acts as an interface to the YouTube API, providing methods for interacting with
    the YouTube V3 API.

    Attributes
    ----------
    client_secret_file: str
        The path to the json file containing your authentication information.
    """

    client_secret_file: Optional[str] = None
    youtube_client: Optional[Any] = None

    def authenticate(self, client_secret_file: Optional[str] = None) -> None:
        """Authenticate the requests made to youtube.

        Used to generate the credentials that are used when authenticating requests to youtube.

        Parameters
        ----------
        client_secret_file: str
            The path to clients secret json file from Google

        Raises
        ------
        ValueError:
            When the client secrets file is not provided
        FileNotFoundError:
            When the secrets file path is not found
        """
        if client_secret_file:
            self.client_secret_file = client_secret_file
        if not self.client_secret_file:
            raise MissingClientSecretsFile("The client secret file must be provided.")
        if not Path(self.client_secret_file).exists():
            raise FileNotFoundError("The client secret file was not found.")
        api_service_name: str = "youtube"
        api_version: str = "v3"
        credentials_dir: str = ".youtube"
        scopes: list[str] = [
            YouTubeScopes.youtube.value,
            YouTubeScopes.youtube_force_ssl.value,
            YouTubeScopes.youtube_upload.value,
        ]
        oauth: GoogleOAuth = GoogleOAuth(
            secrets_file=self.client_secret_file,
            scopes=scopes,
            api_service_name=api_service_name,
            api_version=api_version,
            credentials_dir=credentials_dir,
        )
        self.youtube_client = oauth.authenticate_google_server()
        try:
            self.youtube_client = oauth.authenticate_google_server()
        except InvalidSecretsFileException as e:
            raise InvalidSecretsFileError(e)

    def find_video_by_id(self, video_id: str) -> Video:
        """Find a single video by providing the video's id.

        Parameters
        ----------
        video_id: str
            The video's id
        Returns
        -------
        Video:
            A Video instance
        """
        if not video_id:
            raise ValueError("The video id was not provided")
        video_resource: YouTubeVideoResource = YouTubeVideoResource(
            youtube_client=self.youtube_client
        )
        video: Video = video_resource.find_video_by_id(video_id=video_id)
        return video

    def find_videos_by_ids(self, video_ids: list[str]) -> list[Video]:
        """Find a many videos by providing a list of video ids.

        Parameters
        ----------
        video_ids: list[str]
            A list of video ids
        Returns
        -------
        list[Video]:
            A list of Video instances
        """
        if not video_ids:
            raise ValueError("The video id was not provided")
        video_resource: YouTubeVideoResource = YouTubeVideoResource(
            youtube_client=self.youtube_client
        )
        videos: list[Video] = video_resource.find_videos_by_ids(video_ids=video_ids)
        return videos



from .page_info import PageInfo
from .resource import Resource
from .response import Response
from .video import Video
from .video_list_response import VideoListResponse
from .localized import Localized
from .thumbnail import Thumbnail
from .snippet import Snippet
from .statistics import Statistics
from .status import Status
from .paid_product_placement_details import PaidProductPlacementDetails
from .player import Player
from .topic_details import TopicDetails
from .recording_details import RecordingDetails
from .file_details import FileDetails
from .processing_details import ProcessingDetails
from .suggestions import Suggestions
from .livestreaming_details import LiveStreamingDetails
from .localizations import Localizations



from datetime import datetime
from enum import Enum, auto
from typing import Optional

from pydantic import BaseModel, Field

from .request_part import Part
from .request_filter import Filter
from .request_optional_parameters import OptionalParameters


class VideoPart(Part):
    part: list[str] = Field(
        default=[
            "contentDetails",
            "id",
            "liveStreamingDetails",
            "localizations",
            "player",
            "recordingDetails",
            "snippet",
            "statistics",
            "status",
            "topicDetails",
        ]
    )


class VideoFilter(Filter):
    chart: Optional[str] = ""
    id: Optional[list[str]] = Field(default_factory=list)
    myRating: Optional[str] = ""


class VideoOptionalParameters(OptionalParameters):
    h1: Optional[str] = ""
    maxHeight: Optional[int] = None
    maxResults: Optional[int] = None
    maxWidth: Optional[int] = None
    onBehalfOfContentOwner: Optional[str] = ""
    pageToken: Optional[str] = ""
    regionCode: Optional[str] = ""
    videoCategoryId: Optional[str] = ""



from pydantic import BaseModel, Field
from typing import Optional


class Statistics(BaseModel):
    viewCount: Optional[int] = Field(default=None)
    likeCount: Optional[int] = Field(default=None)
    dislikeCount: Optional[int] = Field(default=None)
    favoriteCount: Optional[int] = Field(default=None)
    commentCount: Optional[int] = Field(default=None)
